 /*====================================================================
	Copyright (C) ROCKEY Tech. Co. Ltd. All rights reserved.

	File: 	ET299Test.c
	
	Purpose:Sample console editor for ET299 	
======================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "et299api.h"

int g_hToken = -1;

void ShowBanner()
{
	printf("\nET99 Key Editor v1.0.\nCreate at " __DATE__ ", " __TIME__ "\n\n" );

	printf( "Library version is V1.0\n");
}

unsigned int ShowErrInfo(unsigned int retval)
{
	switch(retval)
	{
	case ET_SUCCESS					 :
		break;
	case ET_ACCESS_DENY					 :
		printf("Err: access denied, have not enough right!\n");
		printf("Press Enter to quit!\r\n");
		
		getchar();
		break;
	case ET_COMMUNICATIONS_ERROR		 :
		printf("Err: have not open the device\n");
		printf("Press Enter to quit!\r\n");
		
		getchar();
		break;
	case ET_INVALID_PARAMETER		     :
		printf("Err: invalid parameter!\n");
		printf("Press Enter to quit!\r\n");
		
		getchar();
		break;
	case ET_NOT_SET_PID					 :
		printf("Err: have not set PID!\n");
		printf("Press Enter to quit!\r\n");
		
		getchar();
		break;
	case ET_UNIT_NOT_FOUND			     :
		printf("Err: open the device fail!\n");
		printf("Press Enter to quit!\r\n");
		
		getchar();
		break;
	case 0xF0                            :
		printf("Err: PIN  have been locked!\n");
		printf("Press Enter to quit!\r\n");
		
		getchar();
		break;
	default								 :
		{
			if(retval&0xF0)
			  {
				printf("Err: PIN is wrong!\n");
			printf("Press Enter to quit!\r\n");
		
			getchar();
			  }
			else
			  {
				printf("Err: unknown error occured!\n");
				printf("Press Enter to quit!\r\n");
		
				getchar();
			  }
		}
		break;
	}
	if(retval != 0)
	{
		printf("(0x%x)\n",retval);
		return retval;	
	}
}
typedef struct tm SYSTEMTIME;
void GetLocalTime(SYSTEMTIME *st)
{
    struct tm *pst = NULL;
    time_t t = time(NULL);
    pst = localtime(&t);
    memcpy(st,pst,sizeof(SYSTEMTIME));
    st->tm_year += 1900;
}
void ShowBinHex(unsigned char* pBin, long len)
{
	// Show 16 bytes each line.
	int  i, j ,k, kk;
	long lLines = len / 16;
	long lCharInLastLine = 0;
	if(0 != len % 16)
	{
		lCharInLastLine = len - lLines * 16;
	}

	for(i = 0; i < lLines; ++i)
	{
		for(j = 0; j < 16; ++j)
		{
			printf("%02X ", pBin[16 * i + j]);

			if(j == 7)
				printf("- ");
		}

		printf("    ");

		for(j = 0; j < 16; ++j)
		{
			if(isprint(pBin[16 * i + j]))
				printf("%c", pBin[16 * i + j]);
			else
				printf(".");
		}

		printf("\n");
	}

	if(0 != lCharInLastLine)
	{
		for(j = 0; j < lCharInLastLine; ++j)
		{
			printf("%02X ", pBin[16 * i + j]);

			if(j == 7 && lCharInLastLine > 8)
				printf("- ");
		}

		k = 0;

		k += ((16 - lCharInLastLine) * 3);

		if(lCharInLastLine <= 8)
		{
			k += 2;
		}

		for(kk = 0; kk < k; ++kk)
			printf(" ");

		printf("    ");

		for(j = 0; j < lCharInLastLine; ++j)
		{
			if(isprint(pBin[16 * i + j]))
				printf("%c", pBin[16 * i + j]);
			else
				printf(".");
		}

		printf("\n");
	}
	printf("\n");
}
// ***************      ET_299 TEST   *********************

unsigned int ShowSetupMenu(void)
{
	unsigned int retval = ET_SUCCESS;
	int  len,seedd;
	unsigned char Pin[20] = {0};
	unsigned char newPin[20]= {0};
	unsigned char soPin[20] = {0};
	unsigned char seed[100] = {0};
	unsigned char bSoPINRetries;
	unsigned char bUserPINRetries;
	unsigned char bUserReadOnly;
	
	// Reset UserPIN
	memset(soPin,0,sizeof(soPin));
	sprintf(soPin,"%s","ffffffffffffffff");
	retval = et_ResetPIN(g_hToken,soPin);
	if(ET_SUCCESS != retval)
	{
		printf("Reset UserPIN ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Reset UserPIN Success!!!\n");

	// Set ReCount
	bSoPINRetries = 0;
	bUserPINRetries = 0;
	bUserReadOnly = 0;
	retval = et_SetupToken(g_hToken,bSoPINRetries,bUserPINRetries,bUserReadOnly,0);
	if(ET_SUCCESS != retval)
	{
		printf("Set ReCount ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Set ReCount Success!!!\n");

	// Change UserPIN
	memset(Pin,0,sizeof(Pin));
	memset(newPin,0,sizeof(Pin));
	sprintf(Pin,"%s","ffffffffffffffff");
	sprintf(newPin,"%s","ffffffffffffffff");
	retval = et_ChangeUserPIN(g_hToken,Pin,newPin);
	if(ET_SUCCESS != retval)
	{
		printf("Change UserPIN ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Change UserPIN Success!!!\n");

	//********   Warning_TEST ***********	
/*	// Gen 	 SoPIN
	len = 6;
	seedd = 123456;
	memset(seed, 0,sizeof(seed));
	sprintf(seed,"%d",seedd);
	memset(soPin, 0,sizeof(soPin));
	retval = et_GenSOPIN(g_hToken,len,seed,soPin);
	if(ET_SUCCESS != retval)
	{
		printf("Gen SoPIN ERR\n");
		return ShowErrInfo(retval);
	}
	else	
	{	
		printf("The new SO PIN is : \n");
		for(len = 0;len < 16;len ++)
			printf("%x",soPin[len]);
		printf("********Please remember!!!\n");
	}
*/
}

unsigned int ShowCryptMenu(void)
{
	unsigned int retval = ET_SUCCESS;
	int keyIndex=0; 		//[1,7]
	unsigned char key[100] = {0};
    	unsigned char text[100] = {0}; //random_data for MD5 [0,51]
    	unsigned char Dig1[20] = {0};
	unsigned char Dig2[20] = {0};
    	unsigned char skey[100] = {0}; //secret Key [0,+&&]
	int index = 0;
	unsigned char tDig[20] = {0};
	
	//***MakeHMACKey   HMAC ************
	printf("\n Create key files for HMAC-MD5 authentication.\n");
	index = 1;
	memset(skey,0,sizeof(skey));
	sprintf(skey,"%s","@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
	retval = MD5_HMAC(NULL,0,skey,strlen(skey),key,tDig);
	if(ET_SUCCESS != retval)
	{
		printf("MakeHMACKey ERR\n");
		return ShowErrInfo(retval);
	}
	else	
	{
		printf("HMAC-MD5 key content:\n");
		ShowBinHex(key,32);
	}
	retval = et_SetKey(g_hToken,index,key);
	if(ET_SUCCESS != retval)
	{
		printf("Set Key ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Set Key Success!!!\n");

	printf("\nUsing key for HMAC-MD5 authentication\n\n");
	// User HMAC
	keyIndex = 1;    //[1,7]
	memset(text,0,sizeof(text));
	sprintf(text,"%s","012345678901324567890132456789012345678901234567890"); //[0,51]random_data
	retval = et_HMAC_MD5(g_hToken,keyIndex,strlen(text),text,Dig1);
	if(ET_SUCCESS != retval)
	{
		printf("HMAC_MD5 ERR\n");
		return ShowErrInfo(retval);
	}
	else	ShowBinHex(Dig1,16);

	memset(skey,0,sizeof(skey));
	sprintf(skey,"%s","@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
	retval = MD5_HMAC(text,strlen(text),skey,strlen(skey),key,Dig2);
	if(ET_SUCCESS != retval)
	{
		printf("HMAC_MD5 ERR\n");
		return ShowErrInfo(retval);
	}
	else	
	{	
		printf("Software HMAC-MD5 compute result:\n");
		ShowBinHex(Dig2,16);
	}
	if(memcmp(Dig1,Dig2,16))
		printf("Verify failed!\n");

	else	printf("Verify successfully!\n");
}

int main(int argc, char* argv[])
{
	unsigned int retval = ET_SUCCESS;
	int k =0,count = 0;
	unsigned char pid[20] = {0};
	unsigned char sopin[20] = {0};
	unsigned char SN[16];
	unsigned char seed[60];	
	unsigned char Pin[20] = {0};
	unsigned int offs = 0;
	unsigned int rLen = 0;
	unsigned char rBuff[1024] = {0};
	unsigned int index = 0;	
	unsigned char keyData[40];
	unsigned char ch_buf[16] = "FFFFFFFFFFFFFFFF";
	ShowBanner();
	
	//***** Please Input  PID *****
	//unsigned char pid_test[20] = "b835aacd";
	unsigned char pid_test[20] = "ffc5eb78";
	//unsigned char pid_test[20] = "ffffffff";
	char test [16];
	printf("Press Enter to continue!\r\n");
	memset(test,0,sizeof(test));
        gets(test);

	/* int ch; */
  	/* printf("Press Enter to continue!\r\n"); */
  
  	/* ch = getchar(); */
	/* while(ch != 10){ */
	/*   ch = getchar(); */
	/* } */
	// Find
	retval = et_FindToken(pid_test,&count);
	if(ET_SUCCESS != retval){
		printf("Find ERR\n");
		return ShowErrInfo(retval);
	}
	printf("Find %d key!!!\n",count);

	// Open    
	retval = et_OpenToken(&g_hToken,pid_test,count); //** count : Insert number of the Dongle,Please change..!**
	if(ET_SUCCESS != retval)
	{
		printf("Open ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Open NO.%d Success!\n",count);

	//************ VERIFY_TEST (Login Stat) ***********
	// Verify So
	memset(Pin,0,sizeof(Pin));
	sprintf(Pin,"%s","ffffffffffffffff");
	retval = et_Verify(g_hToken,ET_VERIFY_SOPIN,Pin);
	if(ET_SUCCESS != retval)
	{
		printf("Login S_User ERR..\n");
		return ShowErrInfo(retval);
	}
	else	printf("Login S_User Success!!!\n");

	// **************Gen PID  [Warning:Are you sure you want to do this??]***
	// ****** If PID is "ffffffff" or new,You must Call this ******
/*		
	sprintf(seed,"%d",123456);
	k = 6; //seedLen <=51
	memset(pid, 0, sizeof(pid));
	retval = et_GenPID(g_hToken,k,seed,pid);
	if(ET_SUCCESS != retval)
	{
		printf("Gen PID ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("********The new PID is : %s,please remember!!!\n",pid);
	//****************************************************************
	
/* Login User Test
	// Reset User_mode
	retval = et_ResetSecurityState(g_hToken);
	if(ET_SUCCESS != retval)
	{
		printf("ReLogin User ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("ReLogin User Success!!!\n");

	// Verify User
	memset(Pin,0,sizeof(Pin));
	sprintf(Pin,"%s","ffffffffffffffff");
	retval = et_Verify(g_hToken,ET_VERIFY_USERPIN,Pin);
	if(ET_SUCCESS != retval)
	{
		printf("Login User ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Login User Success!!!\n");
*/

///*	
	// Get HDid
	retval = et_GetSN(g_hToken,SN);
	if(ET_SUCCESS != retval)
	{
		printf("Get HDid ERR\n");
		return ShowErrInfo(retval);
	}
	else
	{
		printf("The SN is : \n");
		ShowBinHex(SN,8);
	}
	// Random
	memset(rBuff,0,sizeof(rBuff));
	retval = et_GenRandom(g_hToken,rBuff);
	if(ET_SUCCESS != retval)
	{
		printf("Get Random ERR\n");
		return ShowErrInfo(retval);
	}
	else	ShowBinHex(rBuff,16);

	//************* LED_TEST **************

	retval = et_TurnOnLED(g_hToken);
	if(ET_SUCCESS != retval)
	{
		printf("TurnOn LED ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("TurnOn LED Success!!!\n");
	retval = et_TurnOffLED(g_hToken);
	if(ET_SUCCESS != retval)
	{
		printf("TurnOff LED ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("TurnOff LED Success!!!\n");

	//************  MEMORY_TEST *********

	printf(	"  Read   Write   SetKey  \n" );
	// * offset+Len <= 6*1024
	memset(rBuff, 0,sizeof(rBuff));
	// READ..
	offs = 200;
	rLen = 800;
	retval = et_Read(g_hToken,(unsigned short)offs,(unsigned short)rLen,rBuff);
	if(ET_SUCCESS != retval)
	{
		printf("Read Data ERR\n");
		return ShowErrInfo(retval);
	}
	else	
	{
		printf("\n =>> %d (0x%X) byte%s read.\n", rLen, rLen, rLen > 1 ? "s" : "");
		ShowBinHex(rBuff,rLen);
	}	
	// WRITE ..
	offs = 200;
	rLen = 800;
	sprintf(rBuff,"%s","Hello");
	retval = et_Write(g_hToken,(unsigned short)offs,(unsigned short)rLen,rBuff);
	if(ET_SUCCESS != retval)
	{
		printf("Write Data ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("\n =>> %d byte%s written successfully!\n",rLen, rLen > 1 ? "s" : "");

	// SetKey ..
	index = 1; // keyIndex[1,7]
	sprintf(keyData,"%s","HelloHelloHelloHelloHelloHelloHe"); //strlen(keyData) <=32
	retval = et_SetKey(g_hToken,index,keyData);
	if(ET_SUCCESS != retval)
	{
		printf("Set Key ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Set Key Success!!!\n");
	
	// ************** MD5_TEST *************

	retval = ShowCryptMenu();
	// ************** SetUpToken TEST *************

	retval = ShowSetupMenu();
	
	// ************** Turn ON LED *************
	retval = et_TurnOnLED(g_hToken);
	if(ET_SUCCESS != retval)
	{
		printf("TurnOn LED ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("TurnOn LED Success!!!\n");
	//===========
	printf("et_SetKeyEx(ET_KEY_HMAC_MD5, 1, \"FFFFFFFFFFFFFFFF\") ......");	
	retval = et_SetKeyEx(g_hToken, ET_KEY_HMAC_MD5, 1, ch_buf );
	if(retval != ET_SUCCESS)
	{
        	if(retval == 1)
		{
         		printf("Error: %x  ... Need Input SOPIN to Access\n", retval);
       		}
		else
		{
        		printf("Error: %x\n", retval);
		}
	   	return 0;
	}
	printf("OK\n");

	//===========
   printf("et_SetKeyEx(ET_KEY_TDES, 1, \"FFFFFFFFFFFFFFFF\") ......");	
	retval = et_SetKeyEx(g_hToken, ET_KEY_TDES, 1, ch_buf);
	if(retval != ET_SUCCESS)
	{
        	printf("Error: %x\n", retval);
	   	return 0;
	}
	printf("OK\n");
	// ******** Close *************
	retval = et_CloseToken(g_hToken);
	if(ET_SUCCESS != retval)
	{
		printf("Close ERR\n");
		return ShowErrInfo(retval);
	}
	else	printf("Test END......!!!\n");
	printf("Press Enter to quit!\r\n");
		
	getchar();
	return 0;
}


